@@ -146,7 +146,7 @@ class ImageSource(object):
                     pass
                 raise
         return self.source
-    def as_buffer(self, format=None):
+    def as_buffer(self, format=None, paletted=None):
         """
         Returns the image as a file object.
         
@@ -158,7 +158,7 @@ class ImageSource(object):
             if not format:
                 format = self.format
             log.debug('image -> buf(%s)' % (format,))
-            return img_to_buf(self.source, format)
+            return img_to_buf(self.source, format, paletted=paletted)
         if isinstance(self.source, basestring):
             log.debug('file(%s) -> buf' % self.source)
             return open(self.source, 'rb')
@@ -205,19 +205,48 @@ class ReadBufWrapper(object):
             self.stringio = StringIO(self.readbuf.read())
         return getattr(self.stringio, name)
 
-def img_to_buf(img, format='png'):
-    if format == 'png8':
-        img = img.convert('RGB')
-        img = img.convert('P', palette=Image.ADAPTIVE, dither=0)
+def img_to_buf(img, format='png', paletted=None):
+    defaults = {}    
+    if paletted is None:
+        if format == 'png8':
+            paletted = True
+        else:
+            paletted = base_config().image.paletted
+    if paletted:
+        if format in ('png', 'gif', 'png8'):
+            if img.mode == 'RGBA':
+                img = quantize(img, alpha=True, defaults=defaults)
+            else:
+                img = quantize(img)
+            if hasattr(Image, 'RLE'):
+                defaults['compress_type'] = Image.RLE
     format = filter_format(format)
     buf = StringIO()
-    defaults = {}
     if format == 'jpeg':
+        img = img.convert('RGB')
         defaults['quality'] = base_config().image.jpeg_quality
     img.save(buf, format, **defaults)
     buf.seek(0)
     return buf
 
+def quantize(img, colors=256, alpha=False, defaults=None):
+    if hasattr(Image, 'FASTOCTREE'):
+        if not alpha:
+            img = img.convert('RGB')
+        img = img.quantize(colors, Image.FASTOCTREE)
+    else:
+        if alpha:
+            alpha = img.split()[3]
+            img = img.convert('RGB').convert('P', palette=Image.ADAPTIVE, colors=colors)
+            mask = Image.eval(alpha, lambda a: 255 if a <=128 else 0)
+            img.paste(255, mask)
+            if defaults is not None:
+                defaults['transparency'] = 255
+        else:
+            img = img.convert('RGB').convert('P', palette=Image.ADAPTIVE, colors=colors)
+    
+    return img
+    
 def filter_format(format):
     if format.lower() == 'geotiff':
         format = 'tiff'
@@ -691,6 +720,7 @@ class ImageTransformer(object):
             mesh_div -= 1
         for quad in griddify(dst_quad, mesh_div):
             meshes.append(dst_quad_to_src(quad))
+
         result = src_img.as_image().transform(dst_size, Image.MESH, meshes,
                                               image_filter[self.resampling])
         return ImageSource(result, size=dst_size, transparent=src_img.transparent)
