@@ -19,34 +19,16 @@ from urllib2 import urlopen, URLError, HTTPError
 from BaseHTTPServer import HTTPServer, BaseHTTPRequestHandler
 from contextlib import contextmanager
 
-class StopableHTTPServer(HTTPServer):
-    stopped = False
-    
-    def serve_forever(self):
-        while not self.stopped:
-            self.handle_request()
-    
-    def force_shutdown(self):
-        self.server_close()
-        self.stopped = True
-        self._dummy_request()
-    
-    def _dummy_request(self):
-        try:
-            urlopen('http://%s:%d/' % self.server_address)
-        except HTTPError, e:
-            pass
-        except URLError, e:
-            pass
-
 class ThreadedStopableHTTPServer(threading.Thread):
     def __init__(self, address, requests_responses):
         threading.Thread.__init__(self, **{'group': None})
-        self.httpd = StopableHTTPServer(address, mock_http_handler(requests_responses))
+        self.requests_responses = requests_responses
+        self.shutdown = False
+        self.httpd = HTTPServer(address, mock_http_handler(requests_responses))
     def run(self):
-        self.httpd.serve_forever()
-    def force_shutdown(self):
-        self.httpd.force_shutdown()
+        while self.requests_responses:
+            if self.shutdown: return
+            self.httpd.handle_request()
 
 def mock_http_handler(requests_responses):
     class MockHTTPHandler(BaseHTTPRequestHandler):
@@ -54,19 +36,31 @@ def mock_http_handler(requests_responses):
             try:
                 return self.do_mock_request()
             except AssertionError:
-                self.server.force_shutdown()
+                self.server.shutdown = True
                 raise
         def do_mock_request(self):
             assert len(requests_responses) > 0, 'got unexpected request (%s)' % self.path
             req, resp = requests_responses.pop(0)
+            if req.get('require_basic_auth', False):
+                f = open('out.txt', 'a')
+                print >>f, self.headers
+                if 'Authorization' not in self.headers:
+                    requests_responses.insert(0, (req, resp)) # push back
+                    self.send_response(401)
+                    self.send_header('WWW-Authenticate', 'Basic realm="Secure Area"')
+                    self.end_headers()
+                    self.wfile.write('no access')
+                    return
             if not query_eq(req['path'], self.path):
                 print 'got request      ', self.path
                 print 'excpected request', req['path']
                 assert False, 'got unexpected request (see stdout)'
+            if 'req_assert_function' in req:
+                assert req['req_assert_function'](self)
             self.start_response(resp)
             self.wfile.write(resp['body'])
             if not requests_responses:
-                self.server.force_shutdown()
+                self.server.shutdown = True
             return
         def start_response(self, resp):
             self.send_response(int(resp.get('status', '200')))
@@ -128,10 +122,7 @@ def query_to_dict(query):
 def mock_httpd(address, requests_responses):
     t = ThreadedStopableHTTPServer(address, requests_responses)
     t.start()
-    try:
-        yield
-    finally:
-        t.force_shutdown()
+    yield
 
 def make_wsgi_env(query_string):
         env = {'QUERY_STRING': query_string,
@@ -139,8 +130,3 @@ def make_wsgi_env(query_string):
                'HTTP_HOST': 'localhost',
               }
         return env
-
-if __name__ == '__main__':
-    requests_responses = [('/foo', 'bar'), ('/baz', 'boo')]
-    httpd = StopableHTTPServer(('', 8080), mock_http_handler(requests_responses))
-    httpd.serve_forever()
\ No newline at end of file
